package org.bench4q.agent.scenario;

import java.util.Date;
import java.util.UUID;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.ThreadPoolExecutor.DiscardPolicy;

import org.bench4q.agent.datacollector.DataCollector;
import org.bench4q.agent.datacollector.impl.ScenarioResultCollector;

public class ScenarioContext {
	private static final long keepAliveTime = 10;
	private UUID testId;
	private Date startDate;
	private Date endDate;
	private ThreadPoolExecutor executor;
	private Scenario scenario;
	private boolean finished;
	private DataCollector dataStatistics;

	public UUID getTestId() {
		return testId;
	}

	private void setTestId(UUID testId) {
		this.testId = testId;
	}

	public Date getStartDate() {
		return startDate;
	}

	public void setStartDate(Date saveStartDate) {
		this.startDate = saveStartDate;
	}

	public Date getEndDate() {
		return endDate;
	}

	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}

	public ThreadPoolExecutor getExecutor() {
		return executor;
	}

	public void setExecutorService(ThreadPoolExecutor executor) {
		this.executor = executor;
	}

	public Scenario getScenario() {
		return scenario;
	}

	public void setScenario(Scenario scenario) {
		this.scenario = scenario;
	}

	public boolean isFinished() {
		return finished;
	}

	public void setFinished(boolean finished) {
		this.finished = finished;
	}

	public DataCollector getDataStatistics() {
		return dataStatistics;
	}

	private void setDataStatistics(DataCollector dataStatistics) {
		this.dataStatistics = dataStatistics;
	}

	private ScenarioContext() {
	}

	public static ScenarioContext buildScenarioContext(UUID testId,
			final Scenario scenario, int poolSize) {
		ScenarioContext scenarioContext = new ScenarioContext();
		scenarioContext.setTestId(testId);
		final SynchronousQueue<Runnable> workQueue = new SynchronousQueue<Runnable>();
		ThreadPoolExecutor executor = new ThreadPoolExecutor(poolSize,
				poolSize, keepAliveTime, TimeUnit.MINUTES, workQueue,
				new DiscardPolicy());
		scenarioContext.setScenario(scenario);
		scenarioContext.setStartDate(new Date(System.currentTimeMillis()));
		scenarioContext.setExecutorService(executor);
		scenarioContext.setDataStatistics(new ScenarioResultCollector(testId));
		return scenarioContext;
	}

	public static ScenarioContext buildScenarioContextWithoutScenario(
			UUID testId, int poolSize) {
		ScenarioContext scenarioContext = new ScenarioContext();
		scenarioContext.setTestId(testId);
		final SynchronousQueue<Runnable> workQueue = new SynchronousQueue<Runnable>();
		ThreadPoolExecutor executor = new ThreadPoolExecutor(poolSize,
				poolSize, keepAliveTime, TimeUnit.MINUTES, workQueue,
				new DiscardPolicy());
		scenarioContext.setStartDate(new Date(System.currentTimeMillis()));
		scenarioContext.setExecutorService(executor);
		scenarioContext.setDataStatistics(new ScenarioResultCollector(testId));
		return scenarioContext;
	}

	public ScenarioContext addScenrio(Scenario scenario) {
		this.setScenario(scenario);
		return this;
	}

	/**
	 * Now, I tolerate that if the requiredLoad <
	 * this.getExecutor.getCorePoolSize(), then the excess threads will be
	 * killed when its current task complete
	 * 
	 * @param requiredLoad
	 */
	void updatePopulation(int requiredLoad) {
		this.getExecutor().setCorePoolSize(requiredLoad);
		this.getExecutor().setMaximumPoolSize(requiredLoad);
	}

}
